package emps.strumieniowo;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.stream.Stream;

public class ObslugaCSV {
	private static final String NAGLOWEK = "employee_id;first_name;last_name;job_title;salary;hire_date;department_name;address;postal_code;city;country";
	private static final String SEP = ";";
	
	private ObslugaCSV() {
		// nie wolno tworzyć obiektu
	}

	public static List<Employee> wczytaj(File plik) {
		List<Employee> emps = new ArrayList<>();
		try(Scanner scanner = new Scanner(plik)) {
			scanner.nextLine();
			
			while(scanner.hasNextLine()) {
				String linia = scanner.nextLine();
				Employee emp = wczytajWiersz(linia);
				emps.add(emp);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
		return emps;
	}
	
	public static List<Employee> wczytaj(String sciezka) {
		return wczytaj(new File(sciezka));
	}
	
	public static Stream<Employee> wczytajStrumien(Path path) {
		// Ta metoda zwraca w wyniku "otwarty strumień", z którego będzie można pobierać kolejne obiekty Employee.
		// Dzięki takiemu podejściu dane wszystkich pracowników nie muszą być wczytywane na raz do pamięci (jak w przypadku listy),
		// tylko mogą być przetwarzane jedne po drugim.
		try {
			return Files.lines(path)
					.skip(1)
					//.parallel()
					.map(ObslugaCSV::wczytajWiersz);
		} catch (IOException e) {
			e.printStackTrace();
			return Stream.empty();
		}
	}
	
	public static Stream<Employee> wczytajStrumien(String sciezka) {
		return wczytajStrumien(Paths.get(sciezka));
	}
	
	public static Stream<Employee> wczytajStrumien(File plik) {
		return wczytajStrumien(plik.toPath());
	}
	
	private static Employee wczytajWiersz(String linia) {
		String[] t = linia.split(SEP, 11);
		return new Employee(Integer.parseInt(t[0]), t[1], t[2], t[3],
				Integer.parseInt(t[4]), LocalDate.parse(t[5]), t[6], t[7], t[8], t[9], t[10]);
	}
	
	
	public static void zapisz(List<Employee> lista, File plik) {
		try(PrintWriter out = new PrintWriter(plik)) {
			out.println(NAGLOWEK);
			for (Employee emp : lista) {
				out.print(emp.getEmployeeId());
				out.print(SEP);
				out.print(emp.getFirstName());
				out.print(SEP);
				out.print(emp.getLastName());
				out.print(SEP);
				out.print(emp.getJobTitle());
				out.print(SEP);
				out.print(emp.getSalary());
				out.print(SEP);
				out.print(emp.getHireDate());
				out.print(SEP);
				out.print(emp.getDepartmentName());
				out.print(SEP);
				out.print(emp.getStreetAddress());
				out.print(SEP);
				out.print(emp.getPostalCode());
				out.print(SEP);
				out.print(emp.getCity());
				out.print(SEP);
				out.print(emp.getCountry());
				out.println();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}

	public static void zapisz(List<Employee> lista, String sciezka) {
		zapisz(lista, new File(sciezka));
	}
}
